#include <gba.h>
#include <stdlib.h>
#include <screen_mode.h>
#include "shapes.h"
#include "shapes_configurations.h"
#include "game.h"
#include "game_bg_gfx.h"
#include "game_shapes_gfx.h"
#include <interrupt.h>
#include "intrs.h"



static struct game global_game ;

struct game* init_game (enum mode mode, bool lines_allowed,
			bool columns_allowed)
{
  short x, y ;

  /* Effacement de l'aire de jeu. */
  for (y = 0; y < BOARD_HEIGHT; y++) {
    for (x = 0; x < BOARD_WIDTH; x++) {
      global_game.board[x][y] = Shape_None ;
    }
  }

  for (x = 0; x < BOARD_WIDTH; x++) global_game.h_counts[x] = 0 ;
  for (y = 0; y < BOARD_HEIGHT; y++) global_game.v_counts[y] = 0 ;
  global_game.mode = mode ;
  /* Activation du mode de suivi du temps en cas de jeu standard ou arcade. */
  if (mode == Mode_Standard) global_game.kill_time = KILL_DELAY ;
  else {
    if (mode == Mode_Arcade) global_game.kill_time = ARCADE_START_KILL_DELAY ;
  }
  global_game.lines_allowed = lines_allowed ;
  global_game.columns_allowed = columns_allowed ;
  global_game.handled.shape = (enum shape) (1 + ((rand ()) % SHAPES_NUMBER)) ;
  global_game.handled.orientation = (enum orientation) ((rand ()) % 4) ;
  global_game.score = 0 ;
  global_game.next = (enum shape) (1 + ((rand ()) % SHAPES_NUMBER)) ;
  global_game.player_x = BOARD_WIDTH / 2 ;
  global_game.player_y = BOARD_HEIGHT / 2 ;
  elapsed_seconds = 0 ;

  return (&global_game) ;
}




/* Ca revient  insrer une Shape_Unbreakable sans pour  */
/* autant incrmenter les totaux de ligne et er colonne. */
void send_penalty (struct game *game)
{
  u16 *output_map = SCREEN_BASE_BLOCK_25_ADDR ;
  u16 shape_start_index, map_start_index ;
  short x0, y0, x, y ;

  x0 = (rand ()) % BOARD_WIDTH ;
  y0 = (rand ()) % BOARD_HEIGHT ;
  /* On va parcourir le tableau jusqu' temps de trouver une case libre. */
  /* On commence par parcourir de la position courante vers la fin.      */
  x = x0 ;
  for (y = y0; y < BOARD_HEIGHT; y++) {
    for (; x < BOARD_WIDTH; x++) {
      if (game->board[x][y] == Shape_None) {
	/* On a trouv une place */
	goto found ;
      }     /* Fin de if (game->board[x][y] == Shape_None). */
    }       /* Fin de for (x = x0; x < BOARD_WIDTH; x++). */
    x = 0 ;
  }         /* Fin de for (y = y0; y < BOARD_HEIGHT; y++). */

  /* Si on n'a pas trouv, on recherche depuis le dbut du tableau */
  /* jusqu' la position tire au hasard pralablement.            */
  for (y = 0; y < y0; y++) {
    for (x = 0; x < BOARD_WIDTH; x++) {
      if (game->board[x][y] == Shape_None) {
	goto found ;
      }     /* Fin de if (game->board[x][y] == Shape_None). */
    }       /* Fin de for (x = x0; x < BOARD_WIDTH; x++). */
  }         /* Fin de for (y = y0; y < BOARD_HEIGHT; y++). */
  return  ;

 found:
  if (game->mode == Mode_Standard)
    game->kill_time = elapsed_seconds + KILL_DELAY ;
  game->board[x][y] = Shape_Unbreakable ;
  /* En mme temps on met  jour l'affichage du layer 1. */
  map_start_index =
    (((y * 2) + AREA_BOARD_TOP_Y) * (256/8)) + (AREA_BOARD_TOP_X + (x * 2)) ;
  shape_start_index =
    SHAPE_EMPTY_INDEX + ((SHAPES_NUMBER + 1) * SHAPE_NB_TILES) ;
  map_start_index =
    (((y * 2) + AREA_BOARD_TOP_Y) * (256/8)) + (AREA_BOARD_TOP_X + (x * 2)) ;
  output_map [map_start_index] = (u16) shape_start_index ;
  output_map [map_start_index + 1] = (u16) (shape_start_index + 1) ;
  map_start_index += (256/8) ;
  output_map [map_start_index] = (u16) (shape_start_index + 2) ;
  output_map [map_start_index + 1] = (u16) (shape_start_index + 3) ;
}




void clamp_player_location (struct game *game)
{
  enum orientation tmp_orient ;
  enum shape tmp_shape ;
  struct chain_code *tmp_ccode = NULL ; ;
  short x, y, x_margin, y_margin ;
  short i ;

  tmp_orient = game->handled.orientation ;
  /* On dcrmente le code de la forme car 0 est utilis pour   */
  /* dire "pas de pice" or tous les tableaux de configurations */
  /* sont indexs sur le code des pices, mais  partir de 0.   */
  tmp_shape = game->handled.shape ;
  tmp_ccode =
    (struct chain_code*) rotated_shapes_ccodes[tmp_shape - 1][tmp_orient] ;

  x = game->player_x ;
  y = game->player_y ;

  for (i = 0; i < tmp_ccode->length; i++) {
    /* Correction du x  gauche. */
    if (x < 0) {
      game->player_x += (-x)  ;
      x = 0 ;
    }
    /* Correction du x  droite. */
    x_margin = x - (BOARD_WIDTH - 1) ;
    if (x_margin > 0) {
      game->player_x -= x_margin ;
      x -= x_margin ;
    }
    /* Correction du y en haut. */
    if (y < 0) {
      game->player_y += (-y) ;
      y = 0 ;
    }
    /* Correction du y en bas. */
    y_margin = y - (BOARD_HEIGHT - 1) ;
    if (y_margin > 0) {
      game->player_y -= y_margin ;
      y -= y_margin ;
    }

    x = x + tmp_ccode->codes[i].x_delta ;
    y = y + tmp_ccode->codes[i].y_delta ;
  }
}




/* Pour ne pas allouer sur la pile... */
static bool h_full[BOARD_HEIGHT] ;
static bool v_full[BOARD_WIDTH] ;

void check_for_cleared (struct game *game)
{
  short x, y, i ;
  u16 *output_map = SCREEN_BASE_BLOCK_25_ADDR ;
  u16 map_start_index ;

  if (game->lines_allowed) {
    /* Recherche des lignes pleines. */
    for (y = 0; y < BOARD_HEIGHT; y++) {
      if (game->h_counts[y] == BOARD_WIDTH) {
	/* Ligne remplie. */
	h_full[y] = true ;
	game->h_counts[y] = 0 ;
	/* Destruction graphique. */
	map_start_index =
	  (((y * 2) + AREA_BOARD_TOP_Y) * (256/8)) + AREA_BOARD_TOP_X ;
	for (i = 0; i < BOARD_WIDTH; i++) {
	  /* Destruction dans l'espace de jeu. */
	  game->board[i][y] = Shape_None ;
	  output_map[map_start_index + (i * 2)] = SHAPE_EMPTY_INDEX ;
	  output_map[map_start_index + (i * 2) + 1] = SHAPE_EMPTY_INDEX ;
	  output_map[map_start_index + (i * 2) + (256/8)] = SHAPE_EMPTY_INDEX ;
	  output_map[map_start_index + (i * 2) + 1 + (256/8)] =
	    SHAPE_EMPTY_INDEX ;
	}
      }
      else h_full[y] = false ;
    }
  }

  if (game->columns_allowed) {
    /* Recherche des colonnes pleines. */
    for (x = 0; x < BOARD_WIDTH; x++) {
      if (game->v_counts[x] == BOARD_HEIGHT) {
	/* Colonne remplie. */
	v_full[x] = true ;
	game->v_counts[x] = 0 ;
	/* Destruction graphique. */
	map_start_index =
	  (AREA_BOARD_TOP_Y * (256/8)) + (AREA_BOARD_TOP_X + (x * 2)) ;
	for (i = 0; i < BOARD_HEIGHT; i++) {
	  /* Destruction dans l'espace de jeu. */
	  game->board[x][i] = Shape_None ;
	  output_map[map_start_index + (i * 2) * (256/8)] = SHAPE_EMPTY_INDEX ;
	  output_map[map_start_index + (i * 2) * (256/8) + 1] =
	    SHAPE_EMPTY_INDEX ;
	  output_map[map_start_index + ((i * 2) + 1) * (256/8)] =
	    SHAPE_EMPTY_INDEX ;
	  output_map[map_start_index + ((i * 2)  + 1) * (256/8) + 1] =
	    SHAPE_EMPTY_INDEX ;
	}
      }
      else v_full[x] = false ;
    }
  }

  /* Maintenant, on va dcrmenter les compteurs verticaux des lignes et */
  /* colonnes qui n'taient totalement remplies mais influences par la  */
  /* destruction de celles qui l'taient.                                */
  i = 0 ;   /* Sert de compteur de lignes / colonnes dtruites. */
  for (y = 0; y < BOARD_HEIGHT; y++) {
    if (h_full[y]) {
      /* On a trouv une ligne pleine. */
      i++ ;
      game->score += ((1 << i) * 10) ;
      for (x = 0; x < BOARD_WIDTH; x++) {
	if (!v_full[x]) game->v_counts[x]-- ;
	else {
	  /* Cette ligne croisait une colonne galement pleine. */
	  game->score += 250 ;
	}
      }    /* Fin de for (x = 0; x < BOARD_WIDTH; x++). */
    }      /* Fin de if (h_full[y]). */
  }        /* Fin de for (y = 0; y < BOARD_HEIGHT; y++). */

  for (x = 0; x < BOARD_WIDTH; x++) {
    if (v_full[x]) {
      /* On a trouv une colonne pleine. */
      i++ ;
      game->score += ((1 << i) * 10) ;
      for (y = 0; y < BOARD_HEIGHT; y++) {
	if (!h_full[y]) game->h_counts[y]-- ;
	else {
	  /* Cette colonne croisait une ligne galement pleine. */
	  game->score += 250 ;
	}
      }    /* Fin de for (y = 0; y < BOARD_HEIGHT; y++). */
    }      /* Fin de if (v_full[x]). */
  }        /* Fin de for (x = 0; x < BOARD_WIDTH; x++). */

  if (game->mode == Mode_Arcade)
    game->kill_time += (i * ARCADE_LINES_BONUS) ;
}





static void random_explode (struct game *game)
{
  short nb, i, x, y ;
  u16 map_start_index ;
  u16 *output_map = SCREEN_BASE_BLOCK_25_ADDR ;

  nb = (rand ()) % MAX_EXPLOSIONS ;
  if (nb < MIN_EXPLOSIONS) nb = MIN_EXPLOSIONS ;

  for (i = 0; i < nb; i++) {
    x = (rand ()) % BOARD_WIDTH ;
    y = (rand ()) % BOARD_HEIGHT ;
    if (game->board[x][y] != Shape_None) {
      game->board[x][y] = Shape_None ;
      /* Et en mme temps on met  jour les compteurs. */
      game->v_counts[x]-- ;
      game->h_counts[y]-- ;
      /* Destruction graphique. */
      map_start_index =
	(((y * 2) + AREA_BOARD_TOP_Y) * (256/8)) +
	(AREA_BOARD_TOP_X + (x * 2)) ;
      output_map [map_start_index] = (u16) SHAPE_EMPTY_INDEX ;
      output_map [map_start_index + 1] = (u16) (SHAPE_EMPTY_INDEX + 1) ;
      map_start_index += (256/8) ;
      output_map [map_start_index] = (u16) (SHAPE_EMPTY_INDEX + 2) ;
      output_map [map_start_index + 1] = (u16) (SHAPE_EMPTY_INDEX + 3) ;
    }
  }
}






/* Pour placer une pice sans effet ou une pice avec explosion alatoire. */
static bool try_lay_down_regular (struct game *game, bool explode)
{
  enum orientation current_orient ;
  enum shape current_shape ;
  struct chain_code *current_ccode = NULL ;
  short i, x, y ;
  u16 map_start_index, shape_start_index ;
  u16 *output_map = SCREEN_BASE_BLOCK_25_ADDR ;

  current_shape = game->handled.shape ;
  current_orient = game->handled.orientation ;
  /* On va rcuprer le chain code de la pice actuellement tenue. */
  /* On utilise le code de la forme - 1 car 0 est utilis pour     */
  /* dire "pas de pice" or tous les tableaux de configurations    */
  /* sont indexs sur le code des pices, mais  partir de 0.      */
  current_ccode =
    (struct chain_code*)
    rotated_shapes_ccodes[current_shape - 1][current_orient] ;

  /* On commence par regarder si la pice rentre  l'endroit courant. */
  x = game->player_x ;
  y = game->player_y ;
  for (i = 0; i < current_ccode->length; i++) {
    if ((game->board[x][y]) != Shape_None)
      return (false) ;             /* Espace non autoris. */
    else {
      x = x + current_ccode->codes[i].x_delta ;
      y = y + current_ccode->codes[i].y_delta ;
    }
  }

  /* On fait un coup d'explosion alatoire avant si ncessaire. */
  if (explode) random_explode (game) ;

  /* On a russi  loger la pice, on met donc  jour l'aire de jeu. */
  x = game->player_x ;
  y = game->player_y ;
  shape_start_index = SHAPE_EMPTY_INDEX + (current_shape * SHAPE_NB_TILES) ;
  for (i = 0; i < current_ccode->length; i++) {
    game->board[x][y] = current_shape ;
    /* En mme temps on met  jour l'affichage du layer 1. */
    map_start_index =
      (((y * 2) + AREA_BOARD_TOP_Y) * (256/8)) + (AREA_BOARD_TOP_X + (x * 2)) ;
    output_map [map_start_index] = (u16) shape_start_index ;
    output_map [map_start_index + 1] = (u16) (shape_start_index + 1) ;
    map_start_index += (256/8) ;
    output_map [map_start_index] = (u16) (shape_start_index + 2) ;
    output_map [map_start_index + 1] = (u16) (shape_start_index + 3) ;
    /* Et en mme temps on met  jour les compteurs. */
    game->v_counts[x]++ ;
    game->h_counts[y]++ ;
    /* Morceau suivant de la pice. */
    x = x + current_ccode->codes[i].x_delta ;
    y = y + current_ccode->codes[i].y_delta ;
  }

  return (true) ;
}




static void lay_down_yingyang (struct game *game)
{
  enum orientation current_orient ;
  enum shape current_shape ;
  struct chain_code *current_ccode = NULL ;
  short i, x, y ;
  u16 map_start_index, shape_start_index ;
  u16 *output_map = SCREEN_BASE_BLOCK_25_ADDR ;

  current_shape = game->handled.shape ;
  current_orient = game->handled.orientation ;
  /* On va rcuprer le chain code de la pice actuellement tenue. */
  /* On utilise le code de la forme - 1 car 0 est utilis pour     */
  /* dire "pas de pice" or tous les tableaux de configurations    */
  /* sont indexs sur le code des pices, mais  partir de 0.      */
  current_ccode =
    (struct chain_code*)
    rotated_shapes_ccodes[current_shape - 1][current_orient] ;

  /* La pice va forcment rentrer, on met donc  jour l'aire de jeu. */
  x = game->player_x ;
  y = game->player_y ;
  for (i = 0; i < current_ccode->length; i++) {
    if (game->board[x][y] == Shape_None) {
      game->board[x][y] = current_shape ;
      shape_start_index =
	SHAPE_EMPTY_INDEX + (current_shape * SHAPE_NB_TILES) ;
      /* Et en mme temps on met  jour les compteurs. */
      game->v_counts[x]++ ;
      game->h_counts[y]++ ;
    }
    else {
      game->board[x][y] = Shape_None ;
      shape_start_index = SHAPE_EMPTY_INDEX ;
      /* Et en mme temps on met  jour les compteurs. */
      game->v_counts[x]-- ;
      game->h_counts[y]-- ;
    }

    /* En mme temps on met  jour l'affichage du layer 1. */
    map_start_index =
      (((y * 2) + AREA_BOARD_TOP_Y) * (256/8)) + (AREA_BOARD_TOP_X + (x * 2)) ;
    output_map [map_start_index] = (u16) shape_start_index ;
    output_map [map_start_index + 1] = (u16) (shape_start_index + 1) ;
    map_start_index += (256/8) ;
    output_map [map_start_index] = (u16) (shape_start_index + 2) ;
    output_map [map_start_index + 1] = (u16) (shape_start_index + 3) ;

    /* Morceau suivant de la pice. */
    x = x + current_ccode->codes[i].x_delta ;
    y = y + current_ccode->codes[i].y_delta ;
  }
}




static void lay_down_eraser (struct game *game)
{
  enum orientation current_orient ;
  enum shape current_shape ;
  struct chain_code *current_ccode = NULL ;
  short i, x, y ;
  u16 map_start_index, shape_start_index ;
  u16 *output_map = SCREEN_BASE_BLOCK_25_ADDR ;

  current_shape = game->handled.shape ;
  current_orient = game->handled.orientation ;
  /* On va rcuprer le chain code de la pice actuellement tenue. */
  /* On utilise le code de la forme - 1 car 0 est utilis pour     */
  /* dire "pas de pice" or tous les tableaux de configurations    */
  /* sont indexs sur le code des pices, mais  partir de 0.      */
  current_ccode =
    (struct chain_code*)
    rotated_shapes_ccodes[current_shape - 1][current_orient] ;
  shape_start_index = SHAPE_EMPTY_INDEX ;

  /* La pice va forcment rentrer, on met donc  jour l'aire de jeu. */
  x = game->player_x ;
  y = game->player_y ;
  for (i = 0; i < current_ccode->length; i++) {
    if (game->board[x][y] != Shape_None) {
      game->board[x][y] = Shape_None ;
      /* Et en mme temps on met  jour les compteurs. */
      game->v_counts[x]-- ;
      game->h_counts[y]-- ;

      /* En mme temps on met  jour l'affichage du layer 1. */
      map_start_index =
	(((y * 2) + AREA_BOARD_TOP_Y) * (256/8)) +
	(AREA_BOARD_TOP_X + (x * 2)) ;
      output_map [map_start_index] = (u16) shape_start_index ;
      output_map [map_start_index + 1] = (u16) (shape_start_index + 1) ;
      map_start_index += (256/8) ;
      output_map [map_start_index] = (u16) (shape_start_index + 2) ;
      output_map [map_start_index + 1] = (u16) (shape_start_index + 3) ;
    }

    /* Morceau suivant de la pice. */
    x = x + current_ccode->codes[i].x_delta ;
    y = y + current_ccode->codes[i].y_delta ;
  }
}




bool try_lay_down_piece (struct game *game)
{
  if (game->mode == Mode_Arcade) {
    switch (game->handled.effect) {
    case Effect_None:
      if (!try_lay_down_regular (game, false)) return (false) ;
      break ;

    case Effect_YingYang:
      lay_down_yingyang (game) ;
      break ;

    case Effect_Eraser:
      lay_down_eraser (game) ;
      break ;

    case Effect_Random:
      if (!try_lay_down_regular (game, true)) return (false) ;
      break ;
    }
  }
  else {
    if (!try_lay_down_regular (game, false)) return (false) ;
  }

  /* Vrification des destructions ventuellement engendres par la pose. */
  check_for_cleared (game) ;
  
  /* R-armement du temps en fonction du mode de jeu. */
  switch (global_game.mode) {
  case Mode_Standard: game->kill_time = elapsed_seconds + KILL_DELAY ; break ;
  case Mode_Arcade: game->kill_time += ARCADE_SHAPE_BONUS ; break ;
  default: break ;
  }

  /* On gnre une nouvelle pice. */
  global_game.handled.shape = global_game.next ;
  global_game.handled.orientation = (enum orientation) ((rand ()) % 4) ;
  global_game.next = (enum shape) (1 + ((rand ()) % SHAPES_NUMBER)) ;

  /* On rajoute l'ventuel effet par dessus la pice maintenant tenue. */
  if (global_game.mode == Mode_Arcade) {
    if (((rand ()) % 100) < ARCADE_EFFECT_PERCENT)
      global_game.handled.effect =
	(enum effect) (1 + ((rand ()) % EFFECTS_NUMBER)) ;
    else global_game.handled.effect = Effect_None ;
  }

  clamp_player_location (game) ;
  return (true) ;
}




bool l_or_r_button_pressed (bool is_l, struct game *game)
{
  bool ret_val = false ;
  enum orientation tmp_orient ;

  tmp_orient = game->handled.orientation ;
  if (is_l) {
    /* Rotation en sens inverse des aiguille d'une montre. */
    if (tmp_orient == Orient_Up) tmp_orient = Orient_Left ;
    else tmp_orient-- ;
  }
  else {
    /* Rotation en sens des aiguille d'une montre. */
    tmp_orient = (tmp_orient + 1) % 4;
  }

  game->handled.orientation = tmp_orient ;
  ret_val = true ;
  clamp_player_location (game) ;
  return (ret_val) ;
}



/* Retourne 'false' si le dplacement a chou. */
bool move_cursor_down (struct game *game)
{
  short old_x, old_y ;

  old_x = game->player_x ;
  old_y = game->player_y ;
  game->player_y++ ;
  clamp_player_location (game) ;
  if ((old_x != game->player_x) || (old_y != game->player_y)) return (true) ;
  else return (false) ;
}


/* Retourne 'false' si le dplacement a chou. */
bool move_cursor_up (struct game *game)
{
  short old_x, old_y ;

  old_x = game->player_x ;
  old_y = game->player_y ;
  game->player_y-- ;
  clamp_player_location (game) ;
  if ((old_x != game->player_x) || (old_y != game->player_y)) return (true) ;
  else return (false) ;
}



/* Retourne 'false' si le dplacement a chou. */
bool move_cursor_left (struct game *game)
{
  short old_x, old_y ;

  old_x = game->player_x ;
  old_y = game->player_y ;
  game->player_x-- ;
  clamp_player_location (game) ;
  if ((old_x != game->player_x) || (old_y != game->player_y)) return (true) ;
  else return (false) ;
}


/* Retourne 'false' si le dplacement a chou. */
bool move_cursor_right (struct game *game)
{
  short old_x, old_y ;

  old_x = game->player_x ;
  old_y = game->player_y ;
  game->player_x++ ;
  clamp_player_location (game) ;
  if ((old_x != game->player_x) || (old_y != game->player_y)) return (true) ;
  else return (false) ;
}
