#include <gba.h>
#include <stdlib.h>
#include <string.h>
#include <screen_mode.h>
#include "load_gfx_data.h"
#include "game_bg_gfx.h"
#include "game_shapes_gfx.h"
#include "shapes.h"
#include "shapes_configurations.h"
#include "shape_sprite_gfx.h"
#include "large_fonts_gfx.h"
#include "game.h"
#include "key_control.h"
#include "pause.h"
#include <interrupt.h>
#include "intrs.h"
#include "wav_throw_shape.h"
#include "wav_deny_shape.h"
#include "wav_place_shape.h"
#include "wav_time_warning.h"



/* 4 sprites ncessaires au maximum. */
#define MAX_NUM_PLAYER_SPRITES 4
static struct sprite player_sprites[MAX_NUM_PLAYER_SPRITES] = {
  { 0, 0, 0, 0 },
  { 0, 0, 0, 0 },
  { 0, 0, 0, 0 },
  { 0, 0, 0, 0 }} ;
static short nb_used_player_sprites = 0 ;





void load_game_gfx (bool was_in_pause)
{
  struct sprite *sprite_video_memory = SPRITES_INFO_MEMORY_ADR ;
  u16 old_blending_mode = REG_BLDMOD ;
  short i ;

  set_screen_mode (0) ;

  if (!was_in_pause) {
    /* Initialisations pour le background 0 (layout de base). */
    set_background0
      (BACKGROUND_PRIORITY_3 |            /* Priorit la plus faible. */
       BACKGROUND_CHAR_BASE_BLOCK_0 |     /* Tiles */
       BACKGROUND_COLOR_256 |             /* 256 couleurs. */
       BACKGROUND_SCREEN_BASE_BLOCK_24 |   /* Map. */
       TEXT_BACKGROUND_256x256) ;         /* 256 x 256 pixels. */

    /* Chargement des tiles et de la map de fond d'cran de jeu. */
    load_palette_tile_map_256
      (CHAR_BASE_BLOCK_0_ADDR,       /* Tiles dest. */
       SCREEN_BASE_BLOCK_24_ADDR,    /* Map dest. */
       GAME_BG_PALETTE_SIZE,         /* Taille palette. */
       game_bg_palette,              /* Palette src. */
       GAME_BG_NB_TILES,             /* Nb tiles. */
       game_bg_tiles,                /* Tiles src. */
       GAME_BG_MAP_WIDTH,            /* Largeur map. */
       GAME_BG_MAP_HEIGHT,           /* Hauteur map. */
       game_bg_map) ;                /* Map src. */

    /* Initialisations pour le background 1 (cran de jeu dynamique). */
    set_background1
      (BACKGROUND_PRIORITY_2 |            /* Priorit faible. */
       BACKGROUND_CHAR_BASE_BLOCK_1 |     /* Tiles */
       BACKGROUND_COLOR_256 |             /* 256 couleurs. */
       BACKGROUND_SCREEN_BASE_BLOCK_25 |  /* Map. */
       TEXT_BACKGROUND_256x256) ;         /* 256 x 256 pixels. */

    /* Chargement des tiles de dessin des formes. */
    /* Pas de map, pas de palette. */
    load_palette_tile_map_256
      (CHAR_BASE_BLOCK_1_ADDR,  /* Tiles dest. */
       NULL,                    /* Map dest. */
       0,                       /* Taille palette. */
       NULL,                    /* Palette src. */
       GAME_SHAPES_NB_TILES,    /* Nb tiles. */
       game_shapes_tiles,       /* Tiles src. */
       0,                       /* Largeur map. */
       0,                       /* Hauteur map. */
       NULL) ;                  /* Map src. */
    
    /* Initialisations pour le background 2 (cran de scores). */
    set_background2
      (BACKGROUND_PRIORITY_1 |            /* Priorit moyenne. */
       BACKGROUND_CHAR_BASE_BLOCK_2 |     /* Tiles */
       BACKGROUND_COLOR_256 |             /* 256 couleurs. */
       BACKGROUND_SCREEN_BASE_BLOCK_26 |  /* Map. */
       TEXT_BACKGROUND_256x256) ;         /* 256 x 256 pixels. */

    /* Chargement des tiles de fontes de digits. */
    /* Pas de map, pas de palette. */
    load_palette_tile_map_256
      (CHAR_BASE_BLOCK_2_ADDR,  /* Tiles dest. */
       NULL,                    /* Map dest. */
       0,                       /* Taille palette. */
       NULL,                    /* Palette src. */
       LARGE_FONTS_NB_TILES,    /* Nb tiles. */
       large_fonts_tiles,       /* Tiles src. */
       0,                       /* Largeur map. */
       0,                       /* Hauteur map. */
       NULL) ;                  /* Map src. */

    /* Effacement du fond des layers BG1 et 2 (256x256 pixels=>nbr d'u16) . */
    for (i = 0; i < ((256 / 8) * (256 / 8)); i++) {
      SCREEN_BASE_BLOCK_25_ADDR[i] = SHAPE_EMPTY_INDEX ;
      SCREEN_BASE_BLOCK_26_ADDR[i] = FONT_EMPTY_INDEX ;
    }
  }                     /* Fin de if (!was_in_pause). */
  else {
    /* On a juste besoin de recharger les donnes du layer */
    /* de fonts mais aussi sa palette.                     */
    load_palette_tile_map_256
      (CHAR_BASE_BLOCK_2_ADDR,  /* Tiles dest. */
       NULL,                    /* Map dest. */
       GAME_BG_PALETTE_SIZE,    /* Taille palette. */
       game_bg_palette,         /* Palette src. */
       LARGE_FONTS_NB_TILES,    /* Nb tiles. */
       large_fonts_tiles,       /* Tiles src. */
       0,                       /* Largeur map. */
       0,                       /* Hauteur map. */
       NULL) ;                  /* Map src. */
  }                   /* Fin du else de if (!was_in_pause). */


  /* Chargement des sprites reprsentant les formes. */
  load_sprites_256 (SHAPE_SPRITE_LOADING_OFFSET,
		    SHAPE_SPRITE_NB_TILES,
		    shape_sprite_tiles,
		    SHAPE_SPRITE_PALETTE_SIZE,
		    shape_sprite_palette) ;

  /* Cache tous les sprites en dehors de l'cran pour que ceux que l'on */
  /* n'utilisera pas ne se voient pas et on vide le reste de la mmoire */
  /* contenant les attributs des sprites.                               */
  while (REG_VCOUNT < 160) ;
  for (i = 0; i < 128; i++) {
    sprite_video_memory[i].attribute0 = 160 ; /* y vers > 159. */
    sprite_video_memory[i].attribute1 = 240 ; /* x vers  > 239. */
    sprite_video_memory[i].attribute2 = 0 ;
    sprite_video_memory[i].attribute3 = 0 ;
  }

  REG_BG0HOFS = 0 ;
  REG_BG0VOFS = 0 ;
  REG_BG1HOFS = 0 ;
  REG_BG1VOFS = 0 ;
  REG_BG2HOFS = 0 ;
  REG_BG2VOFS = 0 ;

  /* Fading ouvrant. */
  REG_COLEY = 15 ;
  set_blending_mode
    (FADE_MODE_3 | FADE_SOURCE_BG0 | FADE_SOURCE_BG1 | FADE_SOURCE_BG2) ;
  set_screen_mode
    (MODE_0 | BG0_ENABLE | BG1_ENABLE | BG2_ENABLE | OBJ_MAP_1D | OBJ_ENABLE) ;
  for (i = (16 << 4) - 1 ; i > 0 ; i--) {
    while (REG_VCOUNT != 160) ;
    REG_COLEY = (u16) (i >> 4) ;
  }
  set_blending_mode (old_blending_mode) ;
}







void large_print_int (u16 *output_map, u16 tile_base_index, u16 col,
		      u16 line, u8 nb_digits, u32 value)
{
  short column ;
  unsigned int div ;
  unsigned int remain = 0 ;
  u16 tile_on_screen ;

  div = value ;
  /* Affichage sur 'nb_digits' chiffres. */
  for (column = nb_digits - 1; column >= 0; column --) {
    remain = div % 10 ;
    div = div / 10 ;
    tile_on_screen = (line * (256 / 8)) + (col + column) ;
    /* Premire tile du chiffre. */
    output_map[tile_on_screen] =
      (u16) (tile_base_index + ((remain + 1) * 2)) ;
    /* Seconde tile du chiffre. */
    tile_on_screen = ((line + 1) * (256 / 8)) + (col + column) ;
    output_map[tile_on_screen] =
      (u16) (tile_base_index + (((remain + 1) * 2) + 1)) ;
  }
}




static void refresh_timer (u32 decrement)
{
  int hours, minutes, seconds ;
  u32 real_time ;

  real_time =
    (decrement != 0 ? decrement - elapsed_seconds : elapsed_seconds) ;
  hours = real_time / (60 * 60) ;
  minutes = (real_time - (hours * 60 * 60)) / 60 ;
  seconds = real_time - (hours * 60 * 60) - (minutes * 60) ;
  large_print_int (SCREEN_BASE_BLOCK_26_ADDR, FONT_EMPTY_INDEX,
		   TIME_X_OFFSET, TIME_Y_OFFSET, 2, hours) ;
  large_print_int (SCREEN_BASE_BLOCK_26_ADDR, FONT_EMPTY_INDEX,
		   TIME_X_OFFSET + 3, TIME_Y_OFFSET, 2, minutes) ;
  large_print_int (SCREEN_BASE_BLOCK_26_ADDR, FONT_EMPTY_INDEX,
		   TIME_X_OFFSET + 6, TIME_Y_OFFSET, 2, seconds) ;
}




void refresh_player_cursor (struct game *game)
{
  struct sprite *sprite_video_memory = SPRITES_INFO_MEMORY_ADR ;
  enum orientation tmp_orient ;
  enum shape tmp_shape ;
  struct chain_code *tmp_ccode = NULL ; ;
  u16 sprite_tile_index = SHAPE_SPRITE_TILES_INDEX ;
  short i ;
  short tmp_x, tmp_y ;

  if (game->mode == Mode_Arcade) {
    switch (game->handled.effect) {
    case Effect_None: break ;   /* Gr par l'init par dfaut. */
    case Effect_YingYang: sprite_tile_index = SHAPE_YING_YANG_INDEX ; break ;
    case Effect_Eraser: sprite_tile_index = SHAPE_ERASER_INDEX ; break ;
    case Effect_Random:
      sprite_tile_index = SHAPE_RANDOM_EXPLODE_INDEX ; break ;
    }
  }
  /* Sinon, grce  l'init par  dfaut de "sprite_tile_index", on utilisera */
  /* le dessin standard, c'est--dire l'engrenage. */

  for (i = 0; i < MAX_NUM_PLAYER_SPRITES; i++) {
    /* Effacement des anciens X. */
    player_sprites[i].attribute1 = player_sprites[i].attribute1 & 0xFE00 ;
    /* Effacement de l'ancien Y. */
    player_sprites[i].attribute0 = player_sprites[i].attribute0 & 0xFF00 ;
  }

  tmp_shape = game->handled.shape ;
  tmp_orient = game->handled.orientation ;

  /* On dcrmente le code de la forme car 0 est utilis pour   */
  /* dire "pas de pice" or tous les tableaux de configurations */
  /* sont indexs sur le code des pices, mais  partir de 0.   */
  tmp_shape-- ;
  tmp_ccode =
    (struct chain_code*) rotated_shapes_ccodes[tmp_shape][tmp_orient] ;
  /* Nombre de sprites ncessaires. */
  nb_used_player_sprites = tmp_ccode->length ;
  /* On fait en sorte que la pice se trouve dans la zone  */
  /* de jeu au plus prs de la position du joueur.         */
  tmp_x = (AREA_BOARD_TOP_X * 8) + (game->player_x * 16) ;
  tmp_y = (AREA_BOARD_TOP_Y * 8) + (game->player_y * 16) ;
  for (i = 0; i < nb_used_player_sprites; i++) {
    /* Ordonne. */
    player_sprites[i].attribute0 = SPRITE_256_COLOR | SPRITE_SQUARE | tmp_y ;
    /* Abscisse et taille. */
    player_sprites[i].attribute1 = SPRITE_SIZE_16 | tmp_x ;
    /* Tile. */
    player_sprites[i].attribute2 = sprite_tile_index ;
    tmp_x += (16 * tmp_ccode->codes[i].x_delta) ;
    tmp_y += (16 * tmp_ccode->codes[i].y_delta) ;
  }

  /* Cache tous les sprites "curseurs" non utiliss. */
  for (i = nb_used_player_sprites; i < MAX_NUM_PLAYER_SPRITES; i++) {
    player_sprites[i].attribute0 = 160 ; /* y vers > 159. */
    player_sprites[i].attribute1 = 240 ; /* x vers  > 239. */
  }

  /* Attente synchro VBL. */
  while (REG_VCOUNT < 160) ;
  /* Transfert de toutes les donnes sprite "curseurs" en mmoire. */
  memcpy (&sprite_video_memory[0], player_sprites,
	  MAX_NUM_PLAYER_SPRITES * sizeof (struct sprite)) ;
}




static void shake_player_shape (struct game *game)
{
  struct sprite *sprite_video_memory = SPRITES_INFO_MEMORY_ADR ;
  u16 times, i ;
  u16 x, y ;

  if (nb_used_player_sprites == 0) return ;
  
  for (times = 0; times < 10; times++) {
    for (i = 0; i < nb_used_player_sprites; i++) {
      /* Rcupration des anciennes coordonnes du sprite. */
      x = player_sprites[i].attribute1 & 0x01FF ;
      y = player_sprites[i].attribute0 & 0x00FF ;
      /* Effacement des anciens X. */
      player_sprites[i].attribute1 = player_sprites[i].attribute1 & 0xFE00 ;
      /* Effacement de l'ancien Y. */
      player_sprites[i].attribute0 = player_sprites[i].attribute0 & 0xFF00 ;
      x += (4 - ((s16) ((rand ()) % 8))) ;
      y += (4 - ((s16) ((rand ()) % 8))) ;
      player_sprites[i].attribute1 = player_sprites[i].attribute1 | x ;
      player_sprites[i].attribute0 = player_sprites[i].attribute0 | y ;
    }

    while (REG_VCOUNT != 160) ;
    /* Transfert de toutes les donnes sprite "curseurs" en mmoire. */
    memcpy (&sprite_video_memory[0], player_sprites,
	    nb_used_player_sprites * sizeof (struct sprite)) ;
  }

  /* Finalement, restauration du curseur comme il faut. */
  refresh_player_cursor (game) ;
}



static void show_next_shape (struct game *game)
{
  struct chain_code *current_ccode = NULL ;
  enum shape next_shape = game->next ;
  u16 map_start_index ;
  u16 *output_map ;
  short i, x, y ;

  /* Effacement de l'ancien dessin. */
  for (y = 0; y < 4; y++) {
    output_map =
      SCREEN_BASE_BLOCK_25_ADDR +
      ((NEXT_SHAPE_Y_OFFSET + y) * (256 / 8)) + NEXT_SHAPE_X_OFFSET ;
    for (x = 0; x < 8; x++) {
      *output_map = SHAPE_EMPTY_INDEX ;
      output_map++ ;
    }
  }

  /* Nouveau dessin. */
  switch (next_shape) {
  case Shape_Square_1x1:
    current_ccode =
      (struct chain_code*)
      rotated_shapes_ccodes[(Shape_Square_1x1 - 1)][Orient_Up] ;
    x = NEXT_SHAPE_X_OFFSET + 3 ;
    y = NEXT_SHAPE_Y_OFFSET + 1 ;
    break ;
  case Shape_Square_4x4:
    current_ccode =
      (struct chain_code*)
      rotated_shapes_ccodes[(Shape_Square_4x4 - 1)][Orient_Up] ;
    x = NEXT_SHAPE_X_OFFSET + 2 ;
    y = NEXT_SHAPE_Y_OFFSET ;
    break ;
  case Shape_Long:
    /* On affiche la version horizontale (i.e. tourne). */
    current_ccode =
      (struct chain_code*)
      rotated_shapes_ccodes[(Shape_Long - 1)][Orient_Right] ;
    x = NEXT_SHAPE_X_OFFSET + 6 ;
    y = NEXT_SHAPE_Y_OFFSET + 1 ;
    break ;
  case Shape_Junction:
    /* On affiche la version horizontale Up. */
    current_ccode =
      (struct chain_code*)
      rotated_shapes_ccodes[(Shape_Junction - 1)][Orient_Up] ;
    x = NEXT_SHAPE_X_OFFSET + 1 ;
    y = NEXT_SHAPE_Y_OFFSET + 2 ;
    break ;
  case Shape_Stair_1:
    /* On affiche la version horizontale Up (i.e. non tourne). */
    current_ccode =
      (struct chain_code*)
      rotated_shapes_ccodes[(Shape_Stair_1 - 1)][Orient_Up] ;
    x = NEXT_SHAPE_X_OFFSET + 1 ;
    y = NEXT_SHAPE_Y_OFFSET ;
    break ;

  case Shape_Stair_2:
    /* On affiche la version horizontale Up (i.e. non tourne). */
    current_ccode =
      (struct chain_code*)
      rotated_shapes_ccodes[(Shape_Stair_2 - 1)][Orient_Up] ;
    x = NEXT_SHAPE_X_OFFSET + 1 ;
    y = NEXT_SHAPE_Y_OFFSET + 2 ;
    break ;

  case Shape_Corner:
    current_ccode =
      (struct chain_code*)
      rotated_shapes_ccodes[(Shape_Corner - 1)][Orient_Up] ;
    x = NEXT_SHAPE_X_OFFSET + 3 ;
    y = NEXT_SHAPE_Y_OFFSET ;
    break ;

  default: /* On laisse le pointeur  NULL et tant pis. */ break ;
  }

  if (current_ccode != NULL) {
    u16 shape_start_index = SHAPE_EMPTY_INDEX + (next_shape * SHAPE_NB_TILES) ;
    output_map = SCREEN_BASE_BLOCK_25_ADDR ;
    for (i = 0; i < current_ccode->length; i++) {
      /* On met  jour l'affichage du layer 1. */
      map_start_index = (y  * (256 / 8)) + x ;
      output_map [map_start_index] = (u16) shape_start_index ;
      output_map [map_start_index + 1] = (u16) (shape_start_index + 1) ;
      map_start_index += (256 / 8) ;
      output_map [map_start_index] = (u16) (shape_start_index + 2) ;
      output_map [map_start_index + 1] = (u16) (shape_start_index + 3) ;
      /* Morceau suivant de la pice. */
      x = x + (2 * current_ccode->codes[i].x_delta) ;
      y = y + (2 * current_ccode->codes[i].y_delta) ;
    }
  }
}




static bool a_button_pressed (struct game *game)
{
  bool ret_val ;

  stop_timer2 () ;
  ret_val = try_lay_down_piece (game) ;
  if (ret_val) {
    /* Le joueur a russi  poser sa pice. Il n'en tient donc plus. */
    play_sound (g_wav_place_shape) ;
  }
  else {
    /* Le joueur n'a pas russi  poser sa pice on la secoue. */
    // play_sound (g_wav_dontfit) ;
    play_sound (g_wav_deny_shape) ;
    shake_player_shape (game) ;
  }

  start_timer2 () ;
  return (ret_val) ;
}



static bool b_button_pressed (struct game *game)
{
  u16 old_blending_mode = REG_BLDMOD ;
  short i ;

  /* Active uniquement en mode marathon. */
  switch (game->mode) {
  case Mode_Chrono_1:
  case Mode_Chrono_5:
  case Mode_Chrono_10:
    play_sound (g_wav_throw_shape) ;
    /* On gnre une nouvelle pice. */
    game->handled.shape = game->next ;
    game->handled.orientation = (enum orientation) ((rand ()) % 4) ;
    game->next = (enum shape) (1 + ((rand ()) % SHAPES_NUMBER)) ;
    clamp_player_location (game) ;
    return (true) ;
  default:
    /* On flash. Fading ouvrant. */
    play_sound (g_wav_deny_shape) ;
    REG_COLEY = 0 ;
    set_blending_mode
      (FADE_MODE_2 | FADE_SOURCE_BG0 | FADE_SOURCE_BG1 | FADE_SOURCE_BG2) ;
    for (i = 0 ; i < (16 << 4) ; i++) {
      while (REG_VCOUNT != 160) ;
      REG_COLEY = (u16) (i >> 4) ;
    }
    set_blending_mode (old_blending_mode) ;
    return (false) ;
    break ;
  }
}



static void change_player_color (u32 tick)
{
  short i ;
  struct sprite *sprite_video_memory = SPRITES_INFO_MEMORY_ADR ;
  u16 tile_index = SHAPE_SPRITE_TILES_INDEX ;

  if (tick < KILL_DELAY) {
    /* * 2 car les sprites en 256 couleurs prennent 1 tiles pour un bloc */
    /* visible de _x_ pixels au lieu de 1 tile en mode 16 couleurs.      */
    tile_index += tick * (SHAPE_SPRITE_NB_TILES_PER_SHAPE * 2) ;
    for (i = 0; i < nb_used_player_sprites; i++) {
      player_sprites[i].attribute2 = tile_index ;
    }

    /* Attente synchro VBL. */
    while (REG_VCOUNT < 160) ;
    /* Transfert de toutes les donnes sprite "curseurs" en mmoire. */
    memcpy (&sprite_video_memory[0], player_sprites,
	    nb_used_player_sprites * sizeof (struct sprite)) ;
  }
}




void mainloop (struct game *game)
{
  bool end = false ;
  bool moved = false ;
  u32 decrement = 0 ;
  u32 tick ;
  bool warned = false ;

  switch (game->mode) {
  case Mode_Quiet: case Mode_Standard: decrement = 0 ; break ;
  case Mode_Arcade: decrement = game->kill_time ; break ;
  case Mode_Chrono_1: decrement = 60 ; break ;
  case Mode_Chrono_5: decrement = 60 * 5 ; break ;
  case Mode_Chrono_10: decrement = 60 * 10 ; break ;
  }

  large_print_int (SCREEN_BASE_BLOCK_26_ADDR, FONT_EMPTY_INDEX,
		   SCORE_X_OFFSET, SCORE_Y_OFFSET, SCORE_NB_DIGITS,
		   game->score) ;
  refresh_player_cursor (game) ;
  refresh_timer (decrement) ;
  show_next_shape (game) ;
  start_timer2 () ;

  while (!end) {
    /* Gestion du temps. */
    switch (game->mode) {
    case Mode_Quiet:
      break ;        /* Fin de case Mode_Quiet. */

    case Mode_Standard:
      if (elapsed_seconds >= game->kill_time) {
	send_penalty (game) ;
	warned = false ;
      }
      tick =
	(elapsed_seconds < game->kill_time) ?
	(KILL_DELAY - (game->kill_time - elapsed_seconds)) :
	(KILL_DELAY - 1) ;
      if ((tick == (KILL_DELAY - 2)) && (!warned)) {
	play_sound (g_wav_time_warning) ;
	warned = true ;
      }
      change_player_color (tick) ;
      break ;        /* Fin de case Mode_Standard. */

    case Mode_Arcade:
      decrement = game->kill_time ;
      if (elapsed_seconds >= game->kill_time) {
	end = true ;
	goto refresh_gfxs ;
	}
      break ;     /* Fin de cas Mode_Arcade. */ 

    case Mode_Chrono_1:
      if (elapsed_seconds >= (60 * 1)) {
	end = true ;
	goto refresh_gfxs ;
      }
      break ;        /* Fin de case Mode_Chrono_1. */

    case Mode_Chrono_5:
      if (elapsed_seconds >= (60 * 5)) {
	end = true ;
	goto refresh_gfxs ;
      }
      break ;        /* Fin de case Mode_Chrono_5. */

    case Mode_Chrono_10:
      if (elapsed_seconds >= (60 * 10)) {
	end = true ;
	goto refresh_gfxs ;
      }
      break ;        /* Fin de case Mode_Chrono_10. */
    }                /* Fin de switch (game->mode). */

    moved = false ;

    /* Acquisition des touches. */
    KEY_POOL ;

    if (KEY_HIT (BUTTON_A)) {
      if (a_button_pressed (game)) {
	show_next_shape (game) ;
	moved |= true ;
      }
      goto refresh_gfxs ;
    }


    if (KEY_HIT (BUTTON_B)) {
      if (b_button_pressed (game)) {
	show_next_shape (game) ;
	moved |= true ;
      }
      goto refresh_gfxs ;
    }

    if (KEY_HIT (BUTTON_LEFT)) {
      moved |= l_or_r_button_pressed (true, game) ;
      goto refresh_gfxs ;
    }

    if (KEY_HIT (BUTTON_RIGHT)) {
      moved |= l_or_r_button_pressed (false, game) ;
      goto refresh_gfxs ;
    }

    if (KEY_HIT (KEYPAD_DOWN)) {
      moved |= move_cursor_down (game) ;
      goto refresh_gfxs ;
    }

    if (KEY_HIT (KEYPAD_LEFT)) {
      moved |= move_cursor_left (game) ;
      goto refresh_gfxs ;
    }

    if (KEY_HIT (KEYPAD_RIGHT)) {
      moved |= move_cursor_right (game) ;
      goto refresh_gfxs ;
    }

    if (KEY_HIT (KEYPAD_UP)) {
      moved |= move_cursor_up (game) ;
      goto refresh_gfxs ;
    }

    if ((KEY_HIT (BUTTON_SELECT)) || (KEY_HIT (BUTTON_START))) {
      end = ! (show_pause ()) ;
      /* Si on continue aprs la pause, alors il faut rafrachir le jeu. */
      moved |= !end ;
      goto refresh_gfxs ;
    }


  refresh_gfxs:
    /* On ne raffrachit le pointeur que si l'on a effectivement boug. */
    if (moved) {
      large_print_int (SCREEN_BASE_BLOCK_26_ADDR, FONT_EMPTY_INDEX,
		       SCORE_X_OFFSET, SCORE_Y_OFFSET, SCORE_NB_DIGITS,
		       game->score) ;
      refresh_player_cursor (game) ;
    }
    refresh_timer (decrement) ;
  }

  stop_timer2 () ;
}
