#include <gba.h>
#include <stdlib.h>
#include <string.h>
#include <screen_mode.h>
#include "load_gfx_data.h"
#include "pause_gfx.h"
#include "shape_sprite_gfx.h"
#include "cursor_sprite_gfx.h"
#include "shapes.h"
#include "game.h"
#include "ihm.h"
#include "key_control.h"
#include <interrupt.h>
#include "intrs.h"



#define PAUSE_SELECTION_CONTINUE 0
#define PAUSE_SELECTION_ABANDON 1
static struct sprite cursor_sprite = { 0, 0, 0, 0 } ;


static short handle_pause_input ()
{
  u8 cursor_selection, old_cursor_selection ;
  struct sprite *sprite_video_memory = SPRITES_INFO_MEMORY_ADR ;  

  cursor_selection = PAUSE_SELECTION_CONTINUE ;
  for (;;) {
    old_cursor_selection = cursor_selection ;

    /* Le bouton B fait reprendre par dfaut. */
    KEY_POOL ;
    if (KEY_HIT (BUTTON_B)) return (PAUSE_SELECTION_CONTINUE) ;
    if (KEY_HIT (BUTTON_A)) return (cursor_selection) ;

    if ((KEY_HIT (KEYPAD_UP)) || (KEY_HIT (KEYPAD_DOWN))) {
      cursor_selection = (cursor_selection + 1) % 2 ;
    }

    /* On change la position du curseur seulement s'il a boug. */
    if (cursor_selection != old_cursor_selection) {
      u16 y = PAUSE_CURSOR_SPRITE_CONTINUE_Y ;
      switch (cursor_selection) {
      case PAUSE_SELECTION_CONTINUE:
	y = PAUSE_CURSOR_SPRITE_CONTINUE_Y ; break ;
      case PAUSE_SELECTION_ABANDON:
	y = PAUSE_CURSOR_SPRITE_ABANDON_Y ; break ;
      }
      cursor_sprite.attribute0 = cursor_sprite.attribute0 & 0xFF00 ;
      cursor_sprite.attribute0 = cursor_sprite.attribute0 | y ;  /* y */
      /* Transfert des infos de sprite dans la mmoire vido-sprite. */
      while (REG_VCOUNT < 160) ;
      memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;
    }
  }
}






/* Retourne true si l'on continue le jeu. */
bool show_pause ()
{
  struct sprite *sprite_video_memory = SPRITES_INFO_MEMORY_ADR ;
  u16 old_blending_mode = REG_BLDMOD ;
  short i ;

  stop_timer2 () ;

  /* Fading fermant. */
  REG_COLEY = 0 ;
  set_blending_mode
    (FADE_MODE_3 | FADE_SOURCE_BG0| FADE_SOURCE_BG1 | FADE_SOURCE_BG2) ;
  for (i = 0 ; i < (16 << 4) ; i++) {
    while (REG_VCOUNT != 160) ;
    REG_COLEY = (u16) (i >> 4) ;
  }
  set_screen_mode (0) ;
  set_blending_mode (old_blending_mode) ;

  set_screen_mode (0) ;

  /* Initialisations pour le background 3 (layout de base). */
  /* Attention, on crase les tiles des fontes. */
  set_background3
    (BACKGROUND_PRIORITY_0 |            /* Priorit la plus forte. */
     BACKGROUND_CHAR_BASE_BLOCK_2 |     /* Tiles */
     BACKGROUND_COLOR_256 |             /* 256 couleurs. */
     BACKGROUND_SCREEN_BASE_BLOCK_27 |  /* Map. */
     TEXT_BACKGROUND_256x256) ;         /* 256 x 256 pixels. */

  /* Chargement des tiles de l'cran de pause. */
  /* Attention, on crase la palette du jeu. */
  load_palette_tile_map_256
    (CHAR_BASE_BLOCK_2_ADDR,    /* Tiles dest. */
     SCREEN_BASE_BLOCK_27_ADDR, /* Map dest. */
     PAUSE_PALETTE_SIZE,        /* Taille palette. */
     pause_palette,             /* Palette src. */
     PAUSE_NB_TILES,            /* Nb tiles. */
     pause_tiles,               /* Tiles src. */
     PAUSE_MAP_WIDTH,           /* Largeur map. */
     PAUSE_MAP_HEIGHT,          /* Hauteur map. */
     pause_map) ;               /* Map src. */

  /* Chargement du sprite reprsentant le curseur gnral de slection. */
  load_sprites_256 (CURSOR_SPRITE_LOADING_OFFSET,
		    CURSOR_SPRITE_NB_TILES,
		    cursor_sprite_tiles,
		    CURSOR_SPRITE_PALETTE_SIZE,
		    cursor_sprite_palette) ;

  /* Cache tous les sprites en dehors de l'cran pour que ceux que l'on */
  /* n'utilisera pas ne se voient pas et on vide le reste de la mmoire */
  /* contenant les attributs des sprites.                               */
  while (REG_VCOUNT < 160) ;
  for (i = 0; i < 128; i++) {
    sprite_video_memory[i].attribute0 = 160 ; /* y vers > 159. */
    sprite_video_memory[i].attribute1 = 240 ; /* x vers  > 239. */
    sprite_video_memory[i].attribute2 = 0 ;
    sprite_video_memory[i].attribute3 = 0 ;
  }

  /* Initialisation du sprite du curseur en forme de flche. */
  cursor_sprite.attribute0 =
    SPRITE_256_COLOR | SPRITE_SQUARE | PAUSE_CURSOR_SPRITE_CONTINUE_Y ;
  cursor_sprite.attribute1 = SPRITE_SIZE_16 | PAUSE_CURSOR_SPRITE_X ;
  cursor_sprite.attribute2 = CURSOR_SPRITE_TILE_INDEX ;
  while (REG_VCOUNT < 160) ;
  memcpy (&sprite_video_memory[0], &cursor_sprite, sizeof (struct sprite)) ;


  REG_BG3HOFS = 0 ;
  REG_BG3VOFS = 0 ;
  /* Fading ouvrant. */
  REG_COLEY = 15 ;
  set_blending_mode (FADE_MODE_3 | FADE_SOURCE_BG3) ;
  set_screen_mode (MODE_0 | BG3_ENABLE | OBJ_MAP_1D | OBJ_ENABLE) ;
  for (i = (16 << 4) - 1 ; i > 0 ; i--) {
    while (REG_VCOUNT != 160) ;
    REG_COLEY = (u16) (i >> 4) ;
  }
  set_blending_mode (old_blending_mode) ;

  if (handle_pause_input () == PAUSE_SELECTION_CONTINUE) {
    /* On continue le jeu. On demande donc un       */
    /* rechargement partiel des donnes graphiques. */
    load_game_gfx (true) ;
    start_timer2 () ;
    return (true) ;
  }
  else {
    /* On abandonne. */
    return (false) ;
  }
}
