#include <gba.h>
#include <stdlib.h>
#include "shapes.h"
#include "game.h"
#include "ihm.h"
#include "persistent.h"
#include <screen_mode.h>
#include "load_gfx_data.h"
#include "key_control.h"
#include "highscores_quiet_gfx.h"
#include "highscores_quiet_fonts_gfx.h"
#include "highscores_standard_gfx.h"
#include "highscores_standard_fonts_gfx.h"
#include "highscores_arcade_gfx.h"
#include "highscores_arcade_fonts_gfx.h"
#include "highscores_chrono1_gfx.h"
#include "highscores_chrono1_fonts_gfx.h"
#include "highscores_chrono5_gfx.h"
#include "highscores_chrono5_fonts_gfx.h"
#include "highscores_chrono10_gfx.h"
#include "highscores_chrono10_fonts_gfx.h"





bool check_score (struct game *game) {

  short i, j ;
  u32 *score_table = NULL ;

  switch (game->mode) {
  case Mode_Quiet: score_table = &global_highscores.quiet[0] ; break ;
  case Mode_Standard: score_table = &global_highscores.standard[0] ; break ;
  case Mode_Arcade: score_table = &global_highscores.arcade[0] ; break ;
  case Mode_Chrono_1:
    score_table = &global_highscores.chrono_1[0] ;
    break ;
  case Mode_Chrono_5:
    score_table = &global_highscores.chrono_5[0] ;
    break ;
  case Mode_Chrono_10:
    score_table = &global_highscores.chrono_10[0] ;
    break ;
  default: /* On laisse le pointeur  nul, tant pis. */ break ;
  }

  if (score_table != NULL) {
    /* Recherche d'un point d'entre dans la table des highscores. */
    for (i = 0; i < NB_HIGHSCORES; i++) {
      if (game->score > score_table[i]) {
	/* Dcalage des scores infrieurs. */
	for (j = NB_HIGHSCORES-1; j > i; j--)
	  score_table[j] = score_table[j - 1] ;
	score_table[i] = game->score ;
	/* On sauve les nouveaux highscores. */
	save_scores () ;
	return (true) ;
      }
    }
  }

  return (false) ;
}



static void load_highscore_gfx (enum mode mode)
{
  short i ;
  u16 _palette_size = 0, _nb_tiles = 0 ;
  const u8 *_tiles = NULL ;
  const u8 *_fonts_tiles = NULL ;
  const u16 *_palette = NULL ;
  const u16 *_map = NULL ;

  switch (mode) {
  case Mode_Quiet:
    _palette_size = HIGHSCORES_QUIET_PALETTE_SIZE ;
    _nb_tiles = HIGHSCORES_QUIET_NB_TILES ;
    _tiles = highscores_quiet_tiles ;
    _palette = highscores_quiet_palette ;
    _map = highscores_quiet_map ;
    _fonts_tiles = highscores_quiet_fonts_tiles ;
    break ;

  case Mode_Standard:
    _palette_size = HIGHSCORES_STANDARD_PALETTE_SIZE ;
    _nb_tiles = HIGHSCORES_STANDARD_NB_TILES ;
    _tiles = highscores_standard_tiles ;
    _palette = highscores_standard_palette ;
    _map = highscores_standard_map ;
    _fonts_tiles = highscores_standard_fonts_tiles ;
    break ;

  case Mode_Arcade:
    _palette_size = HIGHSCORES_ARCADE_PALETTE_SIZE ;
    _nb_tiles = HIGHSCORES_ARCADE_NB_TILES ;
    _tiles = highscores_arcade_tiles ;
    _palette = highscores_arcade_palette ;
    _map = highscores_arcade_map ;
    _fonts_tiles = highscores_arcade_fonts_tiles ;
    break ;

  case Mode_Chrono_1:
    _palette_size = HIGHSCORES_CHRONO1_PALETTE_SIZE ;
    _nb_tiles = HIGHSCORES_CHRONO1_NB_TILES ;
    _tiles = highscores_chrono1_tiles ;
    _palette = highscores_chrono1_palette ;
    _map = highscores_chrono1_map ;
    _fonts_tiles = highscores_chrono1_fonts_tiles ;
    break ;

  case Mode_Chrono_5:
    _palette_size = HIGHSCORES_CHRONO5_PALETTE_SIZE ;
    _nb_tiles = HIGHSCORES_CHRONO5_NB_TILES ;
    _tiles = highscores_chrono5_tiles ;
    _palette = highscores_chrono5_palette ;
    _map = highscores_chrono5_map ;
    _fonts_tiles = highscores_chrono5_fonts_tiles ;
    break ;

  case Mode_Chrono_10:
    _palette_size = HIGHSCORES_CHRONO10_PALETTE_SIZE ;
    _nb_tiles = HIGHSCORES_CHRONO10_NB_TILES ;
    _tiles = highscores_chrono10_tiles ;
    _palette = highscores_chrono10_palette ;
    _map = highscores_chrono10_map ;
    _fonts_tiles = highscores_chrono10_fonts_tiles ;
    break ;
  }

  /* Chargement des tiles et de la map de l'cran de fin. */
  load_palette_tile_map_256
    (CHAR_BASE_BLOCK_0_ADDR,          /* Tiles dest. */
     SCREEN_BASE_BLOCK_24_ADDR,       /* Map dest. */
     _palette_size,                   /* Taille palette. */
     _palette,                        /* Palette src. */
     _nb_tiles,                       /* Nb tiles. */
     _tiles,                          /* Tiles src. */
     HIGHSCORES_STANDARD_MAP_WIDTH,  /* Largeur map invariable pr les scores */
     HIGHSCORES_STANDARD_MAP_HEIGHT, /* Hauteur map invariable pr les scores */
     _map) ;                          /* Map src. */

  /* Chargement des tiles de fontes de digits. */
  /* Pas de map, pas de palette. */
  load_palette_tile_map_256
    (CHAR_BASE_BLOCK_1_ADDR,            /* Tiles dest. */
     NULL,                              /* Map dest. */
     0,                                 /* Taille palette. */
     NULL,                              /* Palette src. */
     HIGHSCORES_STANDARD_FONTS_NB_TILES,/* Nb tiles invariable pr les scores */
     _fonts_tiles,                      /* Tiles src. */
     0,                                 /* Largeur map. */
     0,                                 /* Hauteur map. */
     NULL) ;                            /* Map src. */

  /* Effacement du fond du layer BG1 (256x256 pixels=>nbr d'u16) . */
  for (i = 0; i < ((256 / 8) * (256 / 8)); i++)
    SCREEN_BASE_BLOCK_25_ADDR[i] = HIGHSCORES_STANDARD_FONTS_EMPTY_INDEX ;
}



void show_highscores (enum mode mode)
{
  u16 old_blending_mode = REG_BLDMOD ;
  u32 *score_table = NULL ;
  short i ;

  set_screen_mode (0) ;
  
  set_background0
    (BACKGROUND_PRIORITY_3 |            /* Priorit la plus faible. */
     BACKGROUND_CHAR_BASE_BLOCK_0 |     /* Tiles */
     BACKGROUND_COLOR_256 |             /* 256 couleurs. */
     BACKGROUND_SCREEN_BASE_BLOCK_24 |   /* Map. */
     TEXT_BACKGROUND_256x256) ;         /* 256 x 256 pixels. */

  /* Initialisations pour le background 1 (scores). */
  set_background1
    (BACKGROUND_PRIORITY_2 |            /* Priorit faible. */
     BACKGROUND_CHAR_BASE_BLOCK_1 |     /* Tiles */
     BACKGROUND_COLOR_256 |             /* 256 couleurs. */
     BACKGROUND_SCREEN_BASE_BLOCK_25 |  /* Map. */
     TEXT_BACKGROUND_256x256) ;         /* 256 x 256 pixels. */

 dirty_change_displayed_scores:
  load_highscore_gfx (mode) ;

  switch (mode) {
  case Mode_Quiet: score_table = &global_highscores.quiet[0] ; break ;
  case Mode_Standard: score_table = &global_highscores.standard[0] ; break ;
  case Mode_Arcade: score_table = &global_highscores.arcade[0] ; break ;
  case Mode_Chrono_1:
    score_table = &global_highscores.chrono_1[0] ;
    break ;
  case Mode_Chrono_5:
    score_table = &global_highscores.chrono_5[0] ;
    break ;
  case Mode_Chrono_10:
    score_table = &global_highscores.chrono_10[0] ;
    break ;
  default: /* On laisse le pointeur  nul, tant pis. */ break ;
  }

  /* Affichage des scores. */
  if (score_table != NULL) {
    for (i = 0; i < NB_HIGHSCORES; i++) {
      large_print_int (SCREEN_BASE_BLOCK_25_ADDR,
		       HIGHSCORES_STANDARD_FONTS_EMPTY_INDEX,
		       HIGHSCORES_STANDARD_SCORE_X_OFFSET,
		       HIGHSCORES_STANDARD_SCORE_Y_OFFSET + (i * 2),
		       10, score_table[i]) ;
    }
  }

  /* Fading ouvrant. */
  REG_COLEY = 15 ;
  set_blending_mode (FADE_MODE_3 | FADE_SOURCE_BG0 | FADE_SOURCE_BG1) ;
  set_screen_mode (MODE_0 | BG0_ENABLE | BG1_ENABLE) ;
  for (i = (16 << 4) - 1 ; i > 0 ; i--) {
    while (REG_VCOUNT != 160) ;
    REG_COLEY = (u16) (i >> 4) ;
  }
  set_blending_mode (old_blending_mode) ;

  for (;;) {
    KEY_POOL ;
    if ((KEY_HIT (BUTTON_A)) || (KEY_HIT (BUTTON_B)) ||
	(KEY_HIT (BUTTON_SELECT)) || (KEY_HIT (BUTTON_START))) break ;

    if ((KEY_HIT (BUTTON_LEFT)) || (KEY_HIT (KEYPAD_LEFT))) {
      if (mode == Mode_Quiet) mode = Mode_Chrono_10 ;
      else mode-- ;
      /* Fading fermant. */
      REG_COLEY = 0 ;
      set_blending_mode (FADE_MODE_3 | FADE_SOURCE_BG0 | FADE_SOURCE_BG1) ;
      for (i = 0 ; i < (16 << 4) ; i++) {
	while (REG_VCOUNT != 160) ;
	REG_COLEY = (u16) (i >> 4) ;
      }
      set_screen_mode (0) ;
      set_blending_mode (old_blending_mode) ;
      goto dirty_change_displayed_scores ;
    }

    if ((KEY_HIT (BUTTON_RIGHT)) || (KEY_HIT (KEYPAD_RIGHT))) {
      mode = (mode + 1) % 6 ;   /* Car 6 types de jeux. */
      /* Fading fermant. */
      REG_COLEY = 0 ;
      set_blending_mode (FADE_MODE_3 | FADE_SOURCE_BG0 | FADE_SOURCE_BG1) ;
      for (i = 0 ; i < (16 << 4) ; i++) {
	while (REG_VCOUNT != 160) ;
	REG_COLEY = (u16) (i >> 4) ;
      }
      set_screen_mode (0) ;
      set_blending_mode (old_blending_mode) ;
      goto dirty_change_displayed_scores ;
    }
  }

  /* Fading fermant. */
  REG_COLEY = 0 ;
  set_blending_mode (FADE_MODE_3 | FADE_SOURCE_BG0 | FADE_SOURCE_BG1) ;
  for (i = 0 ; i < (16 << 4) ; i++) {
    while (REG_VCOUNT != 160) ;
    REG_COLEY = (u16) (i >> 4) ;
  }
  set_screen_mode (0) ;
  set_blending_mode (old_blending_mode) ;
}
