#include <gba.h>
#include <stdlib.h>
#include <string.h>
#include <screen_mode.h>
#include "shapes.h"
#include "game.h"
#include "load_gfx_data.h"
#include "key_control.h"
#include "menu_gfx.h"
#include "menu_flags_gfx.h"
#include "shape_sprite_gfx.h"
#include "cursor_sprite_gfx.h"
#include "scoring.h"



static struct sprite cursor_sprite = { 0, 0, 0, 0 } ;




static void show_flag (u8 line, u8 column, bool value)
{
  u16 tile ;
  u16 *dest_map ;

  tile =
    (value) ? MENU_FLAG_ON_TILE_START_INDEX : MENU_FLAG_OFF_TILE_START_INDEX ;

  dest_map = SCREEN_BASE_BLOCK_24_ADDR + (line * (256 / 8)) + column ;
  *dest_map = tile ;
  *(dest_map + 1) = tile + 1 ;
  dest_map += (256 / 8) ;
  *dest_map = tile + 2 ;
  *(dest_map + 1) = tile + 3 ;
}


#define STARTER_SELECTION_LINES 0
#define STARTER_SELECTION_COLUMNS 1
#define STARTER_SELECTION_QUIET 2
#define STARTER_SELECTION_STANDARD 3
#define STARTER_SELECTION_ARCADE 4
#define STARTER_SELECTION_CHRONO_1 5
#define STARTER_SELECTION_CHRONO_5 6
#define STARTER_SELECTION_CHRONO_10 7
#define STARTER_SELECTION_SCORES 8


static bool handle_starter_input (enum mode *wished_mode, bool *wished_lines,
				  bool *wished_columns)
{
  short i ;
  u16 old_blending_mode = REG_BLDMOD ;
  u8 cursor_selection, old_cursor_selection ;
  struct sprite *sprite_video_memory = SPRITES_INFO_MEMORY_ADR ;

  cursor_selection = STARTER_SELECTION_QUIET ;
  for (;;) {
    old_cursor_selection = cursor_selection ;
    KEY_POOL ;

    if (KEY_HIT (KEYPAD_UP)) {
      if (cursor_selection == STARTER_SELECTION_LINES)
	cursor_selection = STARTER_SELECTION_SCORES ;
      else cursor_selection-- ;
    }

    if (KEY_HIT (KEYPAD_DOWN)) cursor_selection = (cursor_selection + 1) % 9 ;

    if (KEY_HIT (BUTTON_A)) {
      switch (cursor_selection) {
      case STARTER_SELECTION_QUIET:
	*wished_mode = Mode_Quiet ;
	return (true) ;
	break ;
      case STARTER_SELECTION_STANDARD:
	*wished_mode = Mode_Standard ;
	return (true) ;
	break ;
      case STARTER_SELECTION_ARCADE:
	*wished_mode = Mode_Arcade ;
	return (true) ;
	break ;
      case STARTER_SELECTION_CHRONO_1:
	*wished_mode = Mode_Chrono_1 ;
	return (true) ;
	break ;
      case STARTER_SELECTION_CHRONO_5:
	*wished_mode = Mode_Chrono_5 ;
	return (true) ;
	break ;
      case STARTER_SELECTION_CHRONO_10:
	*wished_mode = Mode_Chrono_10 ;
	return (true) ;
	break ;
      case STARTER_SELECTION_LINES:
	if (*wished_columns) {
	  /* Pour ne pas dsactiver  la fois les lignes et les colonnes ! */
	  *wished_lines = !(*wished_lines) ;
	  show_flag (FLAGS_Y, LINES_FLAG_X, *wished_lines) ;
	}
	else {
	  /* On flash. Fading ouvrant. */
	  REG_COLEY = 0 ;
	  set_blending_mode (FADE_MODE_2 | FADE_SOURCE_BG0) ;
	  for (i = 0 ; i < (16 << 4) ; i++) {
	    while (REG_VCOUNT != 160) ;
	    REG_COLEY = (u16) (i >> 4) ;
	  }
	  set_blending_mode (old_blending_mode) ;
	}
	break ;
      case STARTER_SELECTION_COLUMNS:
	if (*wished_lines) {
	  /* Pour ne pas dsactiver  la fois les lignes et les colonnes ! */
	  *wished_columns = !(*wished_columns) ;
	  show_flag (FLAGS_Y, COLUMNS_FLAG_X, *wished_columns) ;
	}
	else {
	  /* On flash. Fading ouvrant. */
	  REG_COLEY = 0 ;
	  set_blending_mode (FADE_MODE_2 | FADE_SOURCE_BG0) ;
	  for (i = 0 ; i < (16 << 4) ; i++) {
	    while (REG_VCOUNT != 160) ;
	    REG_COLEY = (u16) (i >> 4) ;
	  }
	  set_blending_mode (old_blending_mode) ;
	}
	break ;    /* Fin de case STARTER_SELECTION_COLUMNS. */

      case STARTER_SELECTION_SCORES:
	/* Fading fermant. */
	REG_COLEY = 0 ;
	set_blending_mode (FADE_MODE_3 | FADE_SOURCE_BG0) ;
	for (i = 0 ; i < (16 << 4) ; i++) {
	  while (REG_VCOUNT != 160) ;
	  REG_COLEY = (u16) (i >> 4) ;
	}
	set_screen_mode (0) ;
	show_highscores (Mode_Quiet) ;
	/* Pour dire qu'on n'a rien choisi et    */
	/* qu'il faut recharger les gfx du menu. */
	return (false) ;
	break ;    /* Fin de case STARTER_SELECTION_SCORES. */
      }     /* Fin de switch (cursor_selection). */
    }       /* Fin de if (KEY_HIT (BUTTON_A)). */

    /* On change la position du curseur seulement s'il a boug. */
    if (cursor_selection != old_cursor_selection) {
      u16 x = MODE_QUIET_X * 8 ;
      u16 y = MODE_QUIET_Y * 8 ;
      switch (cursor_selection) {
      case STARTER_SELECTION_QUIET: break ; /* Par dfaut par l'init. */
      case STARTER_SELECTION_STANDARD:
	x = MODE_STANDARD_X * 8 ;
	y = MODE_STANDARD_Y * 8 ;
	break ;
      case STARTER_SELECTION_ARCADE:
	x = MODE_ARCADE_X * 8 ;
	y = MODE_ARCADE_Y * 8 ;
	break ;
      case STARTER_SELECTION_CHRONO_1:
	x = MODE_CHRONO_1_X * 8 ;
	y = MODE_CHRONO_1_Y * 8 ;
	break ;
      case STARTER_SELECTION_CHRONO_5:
	x = MODE_CHRONO_5_X * 8 ;
	y = MODE_CHRONO_5_Y * 8 ;
	break ;
      case STARTER_SELECTION_CHRONO_10:
	x = MODE_CHRONO_10_X * 8 ;
	y = MODE_CHRONO_10_Y * 8 ;
	break ;
      case STARTER_SELECTION_LINES:
	x = (LINES_FLAG_X + 1) * 8 ;
	y = FLAGS_Y * 8 ;
	break ;
      case STARTER_SELECTION_COLUMNS:
	x = (COLUMNS_FLAG_X + 1) * 8 ;
	y = FLAGS_Y * 8 ;
	break ;
      case STARTER_SELECTION_SCORES:
	x = SCORES_X * 8 ;
	y = SCORES_Y * 8 ;
	break ;
      }

      cursor_sprite.attribute1 = cursor_sprite.attribute1 & 0xFE00 ;
      cursor_sprite.attribute1 = cursor_sprite.attribute1 | x ;  /* x */
      cursor_sprite.attribute0 = cursor_sprite.attribute0 & 0xFF00 ;
      cursor_sprite.attribute0 = cursor_sprite.attribute0 | y ;  /* y */
      /* Transfert des infos de sprite dans la mmoire vido-sprite. */
      while (REG_VCOUNT < 160) ;
      memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;
    }
  }
}




void show_starter (enum mode *wished_mode, bool *wished_lines,
		   bool *wished_columns)
{
  struct sprite *sprite_video_memory = SPRITES_INFO_MEMORY_ADR ;
  u16 old_blending_mode = REG_BLDMOD ;
  short i ;

 dirty_restart:
  set_screen_mode (0) ;

  set_background0
    (BACKGROUND_PRIORITY_3 |            /* Priorit la plus faible. */
     BACKGROUND_CHAR_BASE_BLOCK_0 |     /* Tiles */
     BACKGROUND_COLOR_256 |             /* 256 couleurs. */
     BACKGROUND_SCREEN_BASE_BLOCK_24 |   /* Map. */
     TEXT_BACKGROUND_256x256) ;         /* 256 x 256 pixels. */

  /* Chargement des tiles et de la map de l'cran de slection. */
  load_palette_tile_map_256
    (CHAR_BASE_BLOCK_0_ADDR,      /* Tiles dest. */
     SCREEN_BASE_BLOCK_24_ADDR,   /* Map dest. */
     MENU_PALETTE_SIZE,           /* Taille palette. */
     menu_palette,                /* Palette src. */
     MENU_NB_TILES,               /* Nb tiles. */
     menu_tiles,                  /* Tiles src. */
     MENU_MAP_WIDTH,              /* Largeur map. */
     MENU_MAP_HEIGHT,             /* Hauteur map. */
     menu_map) ;                  /* Map src. */

  /* Chargement des tiles de flags. */
  load_palette_tile_map_256
    (CHAR_BASE_BLOCK_0_ADDR + (MENU_NB_TILES * (64/2)),   /* Tiles dest. */
     NULL,                        /* Map dest. */
     0,                           /* Taille palette. */
     NULL,                        /* Palette src. */
     MENU_FLAGS_NB_TILES,         /* Nb tiles. */
     menu_flags_tiles,            /* Tiles src. */
     0,                           /* Largeur map. */
     0,                           /* Hauteur map. */
     NULL) ;                      /* Map src. */

  /* Chargement du sprite reprsentant le curseur gnral de slection. */
  load_sprites_256 (CURSOR_SPRITE_LOADING_OFFSET,
		    CURSOR_SPRITE_NB_TILES,
		    cursor_sprite_tiles,
		    CURSOR_SPRITE_PALETTE_SIZE,
		    cursor_sprite_palette) ;

  /* Cache tous les sprites en dehors de l'cran pour que ceux que l'on */
  /* n'utilisera pas ne se voient pas et on vide le reste de la mmoire */
  /* contenant les attributs des sprites.                               */
  while (REG_VCOUNT < 160) ;
  for (i = 0; i < 128; i++) {
    sprite_video_memory[i].attribute0 = 160 ; /* y vers > 159. */
    sprite_video_memory[i].attribute1 = 240 ; /* x vers  > 239. */
    sprite_video_memory[i].attribute2 = 0 ;
    sprite_video_memory[i].attribute3 = 0 ;
  }

  /* Initialisation du sprite du curseur en forme de flche. */
  cursor_sprite.attribute0 =
    SPRITE_256_COLOR | SPRITE_SQUARE | (MODE_QUIET_Y * 8) ;
  cursor_sprite.attribute1 = SPRITE_SIZE_16 | (MODE_QUIET_X * 8) ;
  cursor_sprite.attribute2 = CURSOR_SPRITE_TILE_INDEX ;
  while (REG_VCOUNT < 160) ;
  memcpy (&sprite_video_memory[0], &cursor_sprite, sizeof (struct sprite)) ;

  /* Valeurs par dfaut des sorties. */
  *wished_lines = true ;
  *wished_columns = true ;
  *wished_mode = Mode_Standard ;
  /* Affichage des flags. */
  show_flag (FLAGS_Y, LINES_FLAG_X, true) ;
  show_flag (FLAGS_Y, COLUMNS_FLAG_X, true) ;


  /* Fading ouvrant. */
  REG_COLEY = 15 ;
  set_blending_mode (FADE_MODE_3 | FADE_SOURCE_BG0) ;
  set_screen_mode (MODE_0 | BG0_ENABLE | OBJ_MAP_1D | OBJ_ENABLE) ;
  for (i = (16 << 4) - 1 ; i > 0 ; i--) {
    while (REG_VCOUNT != 160) ;
    REG_COLEY = (u16) (i >> 4) ;
  }
  set_blending_mode (old_blending_mode) ;

  if (!(handle_starter_input (wished_mode, wished_lines, wished_columns)))
    goto dirty_restart ;

  /* Fading fermant. */
  REG_COLEY = 0 ;
  set_blending_mode (FADE_MODE_3 | FADE_SOURCE_BG0) ;
  for (i = 0 ; i < (16 << 4) ; i++) {
    while (REG_VCOUNT != 160) ;
    REG_COLEY = (u16) (i >> 4) ;
  }
  set_screen_mode (0) ;
  set_blending_mode (old_blending_mode) ;
}
